/**********************************************************************
 * Clears screen                                                      *
 * Entry     : Zero flag must be set. NB: This is done in this routine*
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

cls()                                        /*******************************/
                                             /* Call CLS    in MAIN ROM     */
{                                            /*******************************/
  #asm
    xor   A               ;set zero flag
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,00C3H        ;CLS call
    call  001CH           ;perform slot call
    ei
  #endasm
}   
/**********************************************************************
 * Fill VRAM with specified data (see also BIGFIL)                    *
 * Entry     : VRAM adress                                            *
 *             length                                                 *
 *             data                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

filvrm  (address,length,data)                /*******************************/
                                             /* Call FILVRM in MAIN ROM     */
int  address;                                /* [HL] = address              */
int  length;                                 /* [BC] = length               */
char data;                                   /* [A]  = data                 */
{                                            /*******************************/
  #asm
    pop   IY              ;keep return adres
    pop   DE              ;character (data) in E
    pop   BC              ;length
    pop   HL              ;start video ram adress
    push  HL              ;restore stack
    push  BC              ; -- 
    push  DE              ; --
    push  IY              ; --
    ld    A,E             ;load character in A
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0056H        ;FILVRM call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Moves block of memory from memory to VRAM                          *
 * Entry  : memory adress (source)                                    *  
 *          VRAM adress (destination)                                 *
 *          length                                                    *
 * Philips, 20 februari                                               *
 **********************************************************************/

ldirvm  (maddress,vaddress,length)           /*******************************/
                                             /* Call LDIRVM in MAIN ROM     */
int  maddress;                               /* [HL] = memory address       */
int  vaddress;                               /* [DE] = VRAM address         */
int  length;                                 /* [BC] = length               */
{                                            /*******************************/
  #asm
    pop   IY              ;keep return adress
    pop   BC              ;length
    pop   DE              ;VRAM address
    pop   HL              ;memory address
    push  DE              ;restore stack
    push  HL              ; -- 
    push  BC              ; --
    push  IY              ; --
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,005CH        ;LDIRVM call
    call  001CH           ;perform slot call
    ei
  #endasm
}
/**********************************************************************
 * Initializes screen for text mode, screen 0 and sets VDP            *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

initxt()                                     /*******************************/
                                             /* Call INITXT in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,006CH        ;INITXT call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Initializes screen for text mode (32*24), screen 1 and sets VDP    *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

init32()                                     /*******************************/
                                             /* Call INIT32 in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,006FH        ;INIT32 call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Beeps buzzer                                                       *
 * Entry  : none                                                      *
 * Philips, 20 februari 1986                                          *
 **********************************************************************/

beep ()                                      /*******************************/
                                             /* Call BEEP   in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,00C0H        ;BEEP call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Prints a character on graphics screen                              *
 * Entry  : character                                                 *
 * Philips, 20 februari 1986                                          *
 **********************************************************************/

grpprt (character)                           /*******************************/
                                             /* Call GRPPRT in MAIN ROM     */
char character;                              /* [A] = character to print    */
{                                            /*******************************/
  #asm
    pop   IY              ;keep return adress
    pop   BC              ;character to print in [C]
    push  BC              ;restore stack
    push  IY              ; --
    ld    A,C             ;character in [A]
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,008DH        ;GRPPRT call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * sets VDP mode according to SCRMOD (like basic "screen" command)    *
 * Entry  : screen mode                                               *
 * Philips, 20 februari 1986                                          *
 **********************************************************************/

chgmod (mode)                                /*******************************/
                                             /* Call CHGMOD in MAIN ROM     */
int  mode;                                   /* [A ] = screen mode 0..8     */
{                                            /*******************************/
  #asm
    pop   IY              ;keep return adress
    pop   BC              ;screen mode in [C]
    push  BC              ;restore stack
    push  IY              ; --
    ld    A,C             ;set mode in [A]
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,005FH        ;CHGMOD call
    call  001CH           ;perform slot call
    ei
  #endasm
}   
/**********************************************************************
 * Checks line printer status                                         *
 * Entry     : none                                                   *
 * Returns   : 255 and Z-flag reset if printer ready                  *
 *             0 and Z-flag reset if printer not ready                *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

lptstt()                                     /*******************************/
                                             /* Call LPTSTT in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,00A8H        ;LPTSTT call 
    call  001CH           ;perform slot call
    ld    H,0             ;load return value in HL
    ld    L,A
    ei
  #endasm
}  
/**********************************************************************
 * Initializes function key string contents                           *
 * Entry   : none                                                     *
 * Philips, 20 februari 1986                                          *
 **********************************************************************/

inifnk()                                     /*******************************/
                                             /* Call INIFNK in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,003EH        ;INIFNK call
    call  001CH           ;perform slot call
    ei
  #endasm
}   
/**********************************************************************
 * Sets VDP for hi-resolution mode, screen 2                          *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

setgrp()                                     /*******************************/
                                             /* Call SETGRP in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,007EH        ;SETGRP call 
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Disables screen display                                            *
 * Entry  : none                                                      *
 * Philips, 20 februari 1986                                          *
 **********************************************************************/

disscr()                                     /*******************************/
                                             /* Call DISSCR in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0041H        ;DISSCR call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Changes color of screen                                            *
 * Entry   : forground color, background color, border color          *
 * NB : Forground color in  FORCLR  (F3E9)                            *
 *      Background color in BAKCLR  (F3EA)                            *
 *      Border color in     BDRCLR  (F3EB)                            *
 * Philips, 20 februari 1986                                          *
 **********************************************************************/

chgclr(forclr,bckclr,brdclr)                 /*******************************/
                                             /* Call CHGCLR in MAIN ROM     */
int forclr,                                  /*******************************/
    bckclr,
    brdclr;

{    
  #asm
    pop   IX              ;get return adress
    pop   DE              ;load border color in E
    ld    A,E             ;load border color in A
    ld    (0F3EBH),A      ;store border color
    pop   DE              ;load background color in E
    ld    A,E             ;load background color in A
    ld    (0F3EAH),A      ;store background color
    pop   DE              ;load forground color in E
    ld    A,E             ;load forground color in A
    ld    (0F3E9H),A      ;store forground color
    push  DE              ;restore stack
    push  DE              ; --
    push  DE              ; --
    push  IX              ; __
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0062H        ;CHGCLR call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Enables screen display                                             *
 * Entry  : none                                                      *
 * Philips, 20 februari 1986                                          *
 **********************************************************************/

enascr()                                     /*******************************/
                                             /* Call ENASCR in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0044H        ;ENASCR call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Performs non-maskable interrupt procedures                         *
 * Entry   : none                                                     *
 * Philips, 20 februari 1986                                          *
 **********************************************************************/

nmi()                                        /*******************************/
                                             /* Call NMI    in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0066H        ;NMI    call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Moves block of memory from VRAM to memory                          *
 * Entry     : VRAM adress (source)                                   *
 *             memory adress (destination)                            *
 *             length                                                 *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

ldirmv  (vaddress,maddress,length)           /*******************************/
                                             /* Call LDIRMV in MAIN ROM     */
int  maddress;                               /* [HL] = VRAM address         */
int  vaddress;                               /* [DE] = memory adress        */
int  length;                                 /* [BC] = length               */
{                                            /*******************************/
  #asm
    pop   IY              ;keep return adress
    pop   BC              ;length
    pop   DE              ;Memory address
    pop   HL              ;VRAM address
    push  HL              ;restore stack
    push  DE              ; -- 
    push  BC              ; --
    push  IY              ; --
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0059H        ;LDIRMV call
    call  001CH           ;perform slot call
    ei
  #endasm
}   
/**********************************************************************
 * Initializes screen for hi-resolution mode, screen 2 and sets VDP   *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

inigrp()                                     /*******************************/
                                             /* Call INIGRP in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0072H        ;INIGRP call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Initializes screen for milticolor mode, screen 3 and sets VDP      *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

inimlt()                                     /*******************************/
                                             /* Call INIMLT in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0075H        ;INIMLT call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Sets VDP for text mode , screen 0                                  *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

settxt()                                     /*******************************/
                                             /* Call SETTXT in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0078H        ;SETTXT call
    call  001CH           ;perform slot call
    ei
  #endasm
}   
/**********************************************************************
 * Sets VDP for text mode, screen 1                                   *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

sett32()                                     /*******************************/
                                             /* Call SETT32 in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,007BH        ;SETT32 call
    call  001CH           ;perform slot call
    ei
  #endasm
}   
/**********************************************************************
 * Sets VDP for multicolor mode, screen 3                            *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

setmlt()                                     /*******************************/
                                             /* Call SETMLT in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0081H        ;SETMLT call 
    call  001CH           ;perform slot call
    ei
  #endasm
}
/**********************************************************************
 * Initializes all sprites                                            *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

clrspr()                                     /*******************************/
                                             /* Call CLRSPR in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0069H        ;CLRSPR call 
    call  001CH           ;perform slot call
    ei
  #endasm
}   
/**********************************************************************
 * Initializes PSG and static data for play statement                *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

gicini()                                     /*******************************/
                                             /* Call GICINI in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0090H        ;GICINI call 
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Checks and starts the background task for PLAY                     *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

strtms()                                     /*******************************/
                                             /* Call STRTMS in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0099H        ;STRTMS call 
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Erases function key display                                        *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

erafnk()                                     /*******************************/
                                             /* Call ERAFNK in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,00CCH        ;ERAFNK call 
    call  001CH           ;perform slot call
    ei
  #endasm
}   
/**********************************************************************
 * Displays function key display                                      *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

dspfnk()                                     /*******************************/
                                             /* Call DSPFNK in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,00CFH        ;DSPFNK call 
    call  001CH           ;perform slot call
    ei
  #endasm
}   
/**********************************************************************
 * Forces screen to display mode                                      *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

totext()                                     /*******************************/
                                             /* Call TOTEXT in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,00D2H        ;TOTEXT call 
    call  001CH           ;perform slot call
    ei
  #endasm
}   
/**********************************************************************
 * readln                                                             *
 * 06 feb 1986                                                        *
 **********************************************************************/

#include  <stdio.h>

readln (fd,line)
int fd;
char *line;

{    
int  i;
int  c;
  c = fgetc (fd);
  i = 0;
  while ( ( c != EOF) && (c != '\n'))
  {    
    line [i++] = c;
    c = fgetc (fd);
  }  
  line [i] = NULL;
  return (c);
}  
/**********************************************************************
 * Writes data to VDP register                                        *
 * Entry  : registernr. (0 through 23, 32 through 46)                 *  
 *          data (8 bits)                                             *
 * Philips, 20 februari                                               *
 **********************************************************************/

wrtvdp(regnr,data)                           /*******************************/
                                             /* Call WRTVDP in MAIN ROM     */
int  regnr;                                  /* [C ] = registernr.          */
int  data;                                   /* [B ] = data                 */
{                                            /*******************************/
  #asm
    pop   IY              ;keep return adress
    pop   HL              ;load data in L
    pop   BC              ;load registernr in C
    push  BC              ;restore stack
    push  HL              ; --
    push  IY              ; --
    ld    B,L             ;load data in B
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0047H        ;WRTVDP call
    call  001CH           ;perform slot call
    ei
  #endasm
}  

/**********************************************************************
 * Reads VRAM  (see also NRDVRM)                                      *
 * Entry  : VRAM adress                                               *  
 * Returns: data(byte)                                                *
 * Philips, 20 februari                                               *
 **********************************************************************/

rdvrm(vramadress)                            /*******************************/
                                             /* Call RDVRM  in MAIN ROM     */
int  vramadress;                             /* [HL] = vramadress.          */
                                             /* return value in HL          */
{                                            /*******************************/
  #asm
    pop   IY              ;keep return adress
    pop   HL              ;vramadress in HL
    push  HL              ;restore stack
    push  IY              ; --
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,004AH        ;RDVRM  call
    call  001CH           ;perform slot call
    ld    H,0             ;clear H
    ld    L,A             ;load data in L
    ei
  #endasm
}  
/**********************************************************************
 * Writes to VRAM   (see also NWRVRM)                                 *
 * Entry  : VRAM adress                                               *  
 *          data(byte)                                                *
 * Philips, 20 februari                                               *
 **********************************************************************/

wrtvrm(vramadress,data)                      /*******************************/
                                             /* Call WRTVRM in MAIN ROM     */
int vramadress;                              /* [HL] = vramadress.          */
int data;                                    /* [A ] = data                 */
{                                            /*******************************/
  #asm
    pop   IY              ;keep return adress
    pop   BC              ;data in C         
    pop   HL              ;vramadress in HL
    push  BC              ;restore stack
    push  HL              ; --
    push  IY              ; --
    ld    A,C             ;load data in A
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,004DH        ;WRTVRM call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Returns adress of sprite pattern table                             *
 * Entry  : Sprite ID                                                 *  
 * Returns: Adress                                                    *
 * Philips, 20 februari                                               *
 **********************************************************************/

calpat(sprite_id)                            /*******************************/
                                             /* Call CALPAT in MAIN ROM     */
int  sprite_id;                              /* [A ] = sprite ID            */
                                             /*******************************/
{                                           
  #asm
    pop   IY              ;keep return adress
    pop   BC              ;sprite ID in C
    push  BC              ;restore stack
    push  IY              ; --
    ld    A,C             ;sprite ID in A
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0084H        ;CALPAT call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * displayen string op graphics screen met coordinaten positie        *
 * 12 feb 1986                                                        *
 **********************************************************************/

#include  <stdio.h>
#define   GRPACX   0xFCB7
#define   GRPACY   0xFCB9

grpstr(xpos,ypos,tekst)                      /********************************/
char *tekst;                                 /* printen string op graphics sc*/
int  xpos,ypos;                              /* xpos,ypos zijn de coordinaten*/
{                                            /* tekst is een pointer         */
  int i, *xy;                                /********************************/
  xy  = GRPACX;                              /* aanroep GRPPRT voor printen  */
  *xy = xpos;                                /* 1 karakter                   */
  xy  = GRPACY;                              /* in GRPACX en GRPAXY adres    */
  *xy = ypos;                                /* staan de coordinaten         */
  for (i = 0;                                /*                              */
       tekst[i] != NULL;                     /*                              */
       grpprt(tekst[i++]) );                 /*                              */
}                                            /********************************/
/**********************************************************************
 * Returns adress of sprite attribute table                           *
 * Entry  : Sprite ID                                                 *  
 * Returns: Adress                                                    *
 * Philips, 20 februari                                               *
 **********************************************************************/

calatr(sprite_id)                            /*******************************/
                                             /* Call CALATR in MAIN ROM     */
int  sprite_id;                              /* [A ] = sprite ID            */
                                             /*******************************/
{    
  #asm
    pop   IY              ;keep return adress
    pop   BC              ;sprite ID in C
    push  BC              ;restore stack
    push  IY              ; --
    ld    A,C             ;sprite ID in A
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0087H        ;CALATR call
    call  001CH           ;perform slot call
    ei
  #endasm
}   
/**********************************************************************
 * setup active en display pages for screen mode 8                    *
 * 19 feb 1986                                                        *
 **********************************************************************/

#include  <stdio.h>
#define   ACPAGE   0xFAF6
#define   DPPAGE   0xFAF5

page8(moded,modea)                           /*******************************/
int moded,modea;                             /* set a&d   page in screen 8  */
                                             /*                             */
{              	                             /* mode:                       */
  char *pg;                                  /* 0  : pagina 0               */
  int  i,mode;                               /* 1  : pagina 1               */
  for (i = 0; i <= 1; i++)                   /* 2  : toggle paginanr 0/1    */
                                             /* 3  : niet veranderen        */
  {                                          /*******************************/
  if (i==1) {pg = ACPAGE; mode = modea;}     /* set ACPAGE en               */
  else      {pg = DPPAGE; mode = moded;}     /*     DDPAGE adres            */
  switch (mode)                              /*******************************/
    {                                        /*                             */
    case 0: pg[0] = 0; break;                /* mode 0                      */
    case 1: pg[0] = 1; break;                /* mode 1                      */
    case 2: if (pg[0] == 0 ) pg[0] = 1;      /* mode toglle. vraag paginanr */
                             else pg[0] = 0; /* en set pagina               */
            break;                           /*******************************/
    }                                        /*                             */
  }                                          /*                             */
  setpag();                                  /* set pagina's echt naar      */
}                                            /* video ram                   */
                                             /*******************************/
/**********************************************************************
 * Returns current sprite size                                        *
 * Entry  : none                                                      *  
 * Returns: Sprite size (number of bytes)                             *
 * Philips, 20 februari                                               *
 **********************************************************************/

gspsiz()                                     /*******************************/
                                             /* Call GSPSIZ in MAIN ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,008AH        ;GSPSIZ call
    call  001CH           ;perform slot call
    ld    H,0             ;return waarde in HL
    ld    L,A    
    ei
  #endasm
}   
/**********************************************************************
 * SETPAG                                                             *
 * 19 feb 1986                                                        *
 **********************************************************************/

setpag()                                     /*******************************/
                                             /* Call SETPAG in SUB  ROM     */
{                                            /*******************************/
  #asm
    ld    IY,(0FAF7H)     ;SUB  ROM adres
    ld    IX,013DH        ;SETPAG call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * call DOS MAIN ROM                                                  *
 * 14 feb 1986                                                        *
 **********************************************************************/

#include  <stdio.h>

/**********************************************************************/
/* Funktie naam :DOSMAIN                                              */
/*                                                                    */
/* dosmain (func, AF, BC, DE, HL, back)                               */
/*                                                                    */
/* int    func  = functie code                                        */
/* int    AF    = register AF                                         */
/* int    BC    = register BC                                         */
/* int    DE    = register DE                                         */
/* int    HL    = register HL                                         */
/* char   *back = terugkom block  (min 5 bytes)                       */
/*                                                                    */
/*        BLOCK[0] = register A                                       */
/*        BLOCK[1] = register B                                       */
/*        BLOCK[2] = register C                                       */
/*        BLOCK[3] = register D                                       */
/*        BLOCK[4] = register E                                       */
/* de procedure is een funktie een geeft de return waarde HL          */
/**********************************************************************/


dosmain (func, XAF, XBC, XDE, XHL, back)
int  func;
int  XAF, XBC, XDE, XHL;
char *back;
{    
  #asm
    pop   IY              ;onthou return adres
    ld    (ASTACK),IY
    pop   IY              ;back pointer
    ld    (ABACK),IY 
    pop   HL              ;HL register
    pop   DE              ;DE register
    pop   BC              ;BC register
    pop   AF              ;AF register
    pop   IX              ;funktie code
    push  IX              ;restore stack
    push  AF
    push  BC
    push  DE
    push  HL
    ld    IY,(ABACK)
    push  IY
    ld    IY,(ASTACK)
    push  IY
    ld    IY,(0FCC1H)     ;Main ROM adres
    call  0001CH          ;perform slot call
    ei
    ld    IY,(ABACK)      ;pointer naar blok
    ld    (IY+0),A
    ld    (IY+1),B
    ld    (IY+2),C
    ld    (IY+3),D
    ld    (IY+4),E
  #endasm
}   

#asm
ASTACK:    DW
ABACK:     DW
#endasm

/**********************************************************************
 * call DOS SUB  ROM                                                  *
 * 17 feb 1986                                                        *
 **********************************************************************/

#include  <stdio.h>

/**********************************************************************/
/* Funktie naam :DOSSUB                                               */
/*                                                                    */
/* dossub  (func, AF, BC, DE, HL, back)                               */
/*                                                                    */
/* int    func  = functie code                                        */
/* int    AF    = register AF                                         */
/* int    BC    = register BC                                         */
/* int    DE    = register DE                                         */
/* int    HL    = register HL                                         */
/* char   *back = terugkom block  (min 5 bytes)                       */
/*                                                                    */
/*        BLOCK[0] = register A                                       */
/*        BLOCK[1] = register B                                       */
/*        BLOCK[2] = register C                                       */
/*        BLOCK[3] = register D                                       */
/*        BLOCK[4] = register E                                       */
/* de procedure is een funktie een geeft de return waarde HL          */
/**********************************************************************/


dossub (func, XAF, XBC, XDE, XHL, back)
int  func;
int  XAF, XBC, XDE, XHL;
char *back;
{    
  #asm
    pop   IY              ;onthou return adres
    ld    (ASTACK),IY
    pop   IY              ;back pointer
    ld    (ABACK),IY 
    pop   HL              ;HL register
    pop   DE              ;DE register
    pop   BC              ;BC register
    pop   AF              ;AF register
    pop   IX              ;funktie code
    push  IX              ;restore stack
    push  AF
    push  BC
    push  DE
    push  HL
    ld    IY,(ABACK)
    push  IY
    ld    IY,(ASTACK)
    push  IY
    ld    IY,(0FAF7H)     ;SUB ROM adres
    call  0001CH          ;perform slot call
    ei
    ld    IY,(ABACK)      ;pointer naar blok
    ld    (IY+0),A
    ld    (IY+1),B
    ld    (IY+2),C
    ld    (IY+3),D
    ld    (IY+4),E
  #endasm
}   

#asm
ASTACK:    DW
ABACK:     DW
#endasm
/**********************************************************************
 * Set sprite size 8*8                                                *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

spr8()
      
{
  #asm
    ld    A,(0F3E0H)      ;Load A with contents of VDP register 1
    and   11111101B       ;Reset bit 2 for 8*8 sprites
    ld    (0F3E0H),A      ;Restore contents of VDP register 1
    ei
  #endasm
}
/**********************************************************************
 * Set sprite size 16*16                                              *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

spr16()
      
{
  #asm
    ld    A,(0F3E0H)      ;Load A with contents of VDP register 1
    or    00000010B       ;Set bit 2 for 16*16 sprites
    ld    (0F3E0H),A      ;Restore contents of VDP register 1
    ei
  #endasm
}
/**********************************************************************
 * Reads data from PSG register                                       *
 * Entry  : register number                                           *  
 * Returns: data                                                      *  
 * Philips, 20 februari                                               *
 **********************************************************************/


rdpsg(number)                                /*******************************/
                                             /* Call RDPSG  in MAIN ROM     */
int  number;                                 /* [A ] = register number      */
{                                            /*******************************/
  #asm
    pop   IY              ;keep return adress
    pop   BC              ;get register number in C
    push  BC              ;restore stack
    push  IY              ; --
    ld    A,C             ;load register number in A
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0096H        ;RDPSG  call
    call  001CH           ;perform slot call
    ld    H,0             ;load return value in HL
    ld    L,A  
    ei
  #endasm
}  
/**********************************************************************
 * Writes data to PSG register                                        *
 * Entry  : register number, data                                     *  
 * Philips, 20 februari                                               *
 **********************************************************************/

wrtpsg(number,data)                          /*******************************/
                                             /* Call WRTPSG in MAIN ROM     */
int  number;                                 /* [A ] = register number      */
int  data;                                   /* [E ] = data                 */
{                                            /*******************************/
  #asm
    pop   IY              ;keep return adress
    pop   DE              ;get data in E
    pop   BC              ;get register number in C
    push  BC              ;restore stack
    push  DE              ; -- 
    push  IY              ; --
    ld    A,C             ;load register number in A
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0093H        ;WRTPSG call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Locates cursor at specified position                               *
 * Entry  : column,row                                                *  
 * Philips, 20 februari                                               *
 **********************************************************************/

posit (column,row)                           /*******************************/
                                             /* Call POSIT  in MAIN ROM     */
int  column;                                 /* [H ] = column               */
int  row;                                    /* [L ] = row                  */
{                                            /*******************************/
  #asm
    pop   IY              ;keep return adress
    pop   HL	          ;load row in L
    pop   DE              ;load column in E
    push  DE              ;restore stack
    push  HL              ; --
    push  IY              ; --
    ld    H,E             ;load column in H
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,00C6H        ;POSIT call 
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Clears to end of line                                              *
 * Entry  : column number, line number                                *  
 *          Cursor should remain unchanged                            *
 * Philips, 20 februari                                               *
 **********************************************************************/

eol(column,line)                             /*******************************/
                                             /* Call EOL    in MAIN ROM     */
int  column;                                 /* [H ] = column number        */
int  line;                                   /* [L ] = line number          */
{                                            /*******************************/
  #asm
    pop   IY              ;keep return adress
    pop   HL              ;line number in L
    pop   DE              ;column number in E
    push  DE              ;restore stack
    push  HL              ; -- 
    push  IY              ; --
    ld    H,E             ;column number in H
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0168H        ;EOL    call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Fill VRAM with specified data. Same as FILVRM, except :            *
 * FILVRM checks if the current screen mode is 0,1,2,3. If so, it     *
 * behaves like the VDP has only 16kRAM. This is to maintain          *
 * compatibility with MSX-1. BIGFIL, however, does not check the      *
 * screen mode and fills the VRAM just as specified by the parameters.*
 * Entry     : VRAM adress                                            *
 *             length                                                 *
 *             data                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

bigfil(address,length,data)                  /*******************************/
                                             /* Call BIGFIL in MAIN ROM     */
int  address;                                /* [HL] = address              */
int  length;                                 /* [BC] = length               */
char data;                                   /* [A]  = data                 */
{                                            /*******************************/
  #asm
    pop   IY              ;keep return adres
    pop   DE              ;character (data) in E
    pop   BC              ;length
    pop   HL              ;start video ram adress
    push  HL              ;restore stack
    push  BC              ; -- 
    push  DE              ; --
    push  IY              ; --
    ld    A,E             ;load character in A
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,016BH        ;BIGFIL call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Reads VRAM  (see also  RDVRM)                                      *
 * Entry  : VRAM adress                                               *  
 * Returns: data(byte)                                                *
 * Philips, 20 februari                                               *
 **********************************************************************/

nrdvrm(vramadress)                           /*******************************/
                                             /* Call NRDVRM in MAIN ROM     */
int  vramadress;                             /* [HL] = vramadress.          */
                                             /* return value in HL          */
{                                            /*******************************/
  #asm
    pop   IY              ;keep return adress
    pop   HL              ;vramadress in HL
    push  HL              ;restore stack
    push  IY              ; --
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0174H        ;NRDVRM  call
    call  001CH           ;perform slot call
    ld    H,0             ;clear H
    ld    L,A             ;load data in L
    ei
  #endasm
}  
/**********************************************************************
 * Writes to VRAM   (see also WRTVRM)                                 *
 * Entry  : VRAM adress                                               *  
 *          data(byte)                                                *
 * Philips, 20 februari                                               *
 **********************************************************************/

nwrvrm(vramadress,data)                      /*******************************/
                                             /* Call NWRVRM in MAIN ROM     */
int vramadress;                              /* [HL] = vramadress.          */
int data;                                    /* [A ] = data                 */
{                                            /*******************************/
  #asm
    pop   IY              ;keep return adress
    pop   BC              ;data in C         
    pop   HL              ;vramadress in HL
    push  BC              ;restore stack
    push  HL              ; --
    push  IY              ; --
    ld    A,C             ;load data in A
    ld    IY,(0FCC1H)     ;Main ROM adres
    ld    IX,0177H        ;NWRVRM call
    call  001CH           ;perform slot call
    ei
  #endasm
}  
/**********************************************************************
 * Set sprite magnification double size                               *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

sprdbl()
      
{
  #asm
    ld    A,(0F3E0H)      ;Load A with contents of VDP register 1
    or    00000001B       ;Set bit 1 for double size sprites
    ld    (0F3E0H),A      ;Restore contents of VDP register 1
    ei
  #endasm
}
/**********************************************************************
 * Set sprite magnification normal                                    *
 * Entry     : none                                                   *
 * Philips, 20 feb 1986                                               *
 **********************************************************************/

sprnor()
      
{
  #asm
    ld    A,(0F3E0H)      ;Load A with contents of VDP register 1
    and   11111110B       ;Reset bit 1 for normal size sprite
    ld    (0F3E0H),A      ;Restore contents of VDP register 1
    ei
  #endasm
}
